import java.util.ArrayList;
import java.util.List;

import javafx.collections.ObservableList;
import javafx.scene.Group;
import javafx.scene.Node;

/**
 * 3Dパネルクラス
 * <pre>
 * 時間の制御を行い、適宜、ノードのrunメソッドを呼び出します。
 * キーの押下状態を制御します。
 * </pre>
 */
public class ARPanel3D extends Group implements ARConstant {

	/** アプリケーション */
	private		ARApplication		application		= null;

	/**
	 * コンストラクタ
	 * @param application アプリケーション
	 */
	public ARPanel3D(ARApplication application){

		// スーパークラスを呼び出す
		super();

		// アプリケーション保持
		this.application = application;

		// 初期化処理を呼び出す
		init();

	} // end ARPanel3D

	/**
	 * 初期化処理
	 * <pre>
	 * パネルクラスの初期化処理を行います。
	 * </pre>
	 */
	public void init(){

	} // end init

	/**
	 * 開始
	 * <pre>
	 * 当オブジェクトの処理を開始します。
	 * </pre>
	 */
	public void start(){

	}

	/**
	 * 実行
	 * <pre>
	 * タイマーから定期的に呼ばれます。
	 * </pre>
	 */
	synchronized public void run(){

		// アプリケーションのポーズ中フラグ取得メソッド呼び出し
		boolean flgPause = application.isPause();

		// ゲーム中の場合（ポーズ中ではない場合）、
		if(!flgPause){

			//
			// ノード処理
			//

			// ノード一覧を取得
			List<Node> nodeLst = this.getNodeLst();

			// ノード一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// ノード一覧の件数分、ループ
			for(int j = 0; j < countNodeLst; j++){

				// ノードを取得
				Node node = nodeLst.get(j);

				// ダミーノードの場合、
				if(node instanceof ARNodeDummy){

					// 何もしない

				// 星屑の場合、
				}else if(node instanceof ARSpriteStardust){

					// スプライトを取得
					ARSpriteStardust sprite = (ARSpriteStardust)node;

					// スプライトを起動
					sprite.run();

				// スプライトの場合、
				}else if(node instanceof ARSprite){

					// スプライトを取得
					ARSprite sprite = (ARSprite)node;

					// スプライトを起動
					sprite.run();

				}

			} // end for ノード一覧の件数分、ループ

		} // end if ゲーム中の場合（ポーズ中ではない場合）

	} // end run

	/**
	 * お掃除の時間
	 * <pre>
	 * タイマーから定期的に呼ばれます。
	 * </pre>
	 */
	synchronized public void clean(){

		//
		// ノード一覧が多くなった場合は、お掃除をする。（メモリ増加防止）
		// ノードの実行処理とは別にする（処理分散）
		//

		// ノード一覧を取得
		ObservableList<Node> nodeLst = this.getNodeLst();

		// ノード一覧の件数を取得
		int countNodeLst = nodeLst.size();

		// ノード一覧の件数が1000件を超えた場合、
		if(countNodeLst > 1000){

			// 一時的にノード一覧（ワーク）を作成
			ArrayList<Node> nodeLstWk = new ArrayList<Node>(1000);

			// ノード一覧の件数分、ループ
			for(int j = 0; j < countNodeLst; j++){

				// ノードを取得
				Node node = (Node)nodeLst.get(j);

				// ノードがダミーノード以外の場合、
				if(!(node instanceof ARNodeDummy)){

					// ノード一覧（ワーク）に追加
					nodeLstWk.add(node);

				}

			} // end for ノード一覧の件数分、ループ

			// ノード一覧を入れ替える
			nodeLst.setAll(nodeLstWk);

		} // end if ノード一覧の件数が1000件を超えた場合

	} // end clean

	/**
	 * ノード追加
	 * @param node ノード
	 */
	public void addNode(Node node){

		// ノード一覧を取得
		List<Node> nodeLst = this.getNodeLst();

		// ノードを追加
		nodeLst.add(node);

	} // end addNode

	/**
	 * ノード削除
	 * <pre>
	 * ノードを削除します。
	 * 当メソッドでは、ダミーノードに置き換え、runメソッドの中で削除します。
	 * nullに置き換えができない
     * かつ Listクラスにremoveメソッドを多用することで配列コピーを多用されることを避けるため
	 * （重くなることをできるだけ避ける）
	 * </pre>
	 * @param node ノード
	 */
	public void removeNode(Node node){

		// ノード一覧を取得
		List<Node> nodeLst = this.getNodeLst();

		// インデックスを求める
		int index = nodeLst.indexOf(node);

		// インデックスが取得できた場合、
		if(index != -1){

			// ダミーノードに置き換え
			nodeLst.set(index, new ARNodeDummy(this.application));

		}

	} // end removeNode

	/**
	 * ノード一覧取得
	 * @return ObservableList<Node> ノード一覧
	 */
	public ObservableList<Node> getNodeLst(){

		// Listを取得
		ObservableList<Node> nodeLst = this.getChildren();

		// リターン
		return nodeLst;

	} // end getNodeLst

	/**
	 * アプリケーション取得
	 * @return ARApplication アプリケーション
	 */
	public ARApplication getApplication(){

		// アプリケーションをリターン
		return application;

	} // end getApplication

}
